//  Modbus Arduino slave for Mach3 by Ben Bouwer
//  The purpose of this scetch is to link the Arduino digital and analog pins to Mach3 using the Modbus Protocol.
    // To keep things simple I used a register for each pin in the arduino.
    // Each register can be a Digital input, Digital output, Analog input or PWM output register
    // Choose what you want each Arduino Pin to be (Digital input, Digital output, Analog input, PWM output or MPG input) then:
    // Edit the list below.
    // Ensure that you do not use one Arduino pin twice.

//  Recommended Modbus Master: QModbus or Mach3
    // QModbus http://qmodbus.sourceforge.net/ or
     // Mach3
     
//  Mach 3 settings:
//  See: www.machsupport.com/
//  Check out www.youtube.com
//  See: Using Modbus with Mach3 By Peter Homann


// Write a Brain in Mach3 for each register you want to use
    // To use a button on Arduino to start something in Mach3 use:
       // Modbus Register --> Pass through --> Output (eg. Cycle start)
    // To use a LED in Mach3 to switch on a LED on Arduino use:   
       // LED --> Pass through --> Modbus Register
 
//  Analog Input Pins
  //  The Atmega controllers used for the Arduino contain an onboard 6 channel analog-to-digital (A/D) converter. The converter has 10 bit resolution, 
  //  returning integers from 0 to 1023. If the analog pins are used to detect sensor values, these values must be maped to a range that is usable (eg. 0 to 255). 
  //  While the main function of the analog pins for most Arduino users is to read analog sensors, the analog pins also 
  //  have all the functionality of general purpose input/output (GPIO) pins (the same as digital pins 0 - 13). 
  //  Consequently, if a user needs more general purpose input output pins, and all the analog pins are not in use, the analog pins may be used for GPIO.
    
  //  Pin mapping
    //  The analog pins can be used identically to the digital pins, using the aliases A0 (for analog input 0), A1, etc. For example, the code would look like this 
    //  to set analog pin 0 to an output, and to set it HIGH: 
    //  pinMode(A0, OUTPUT);
    //  digitalWrite(A0, HIGH);
    
  //  Pullup resistors
    //  The analog pins also have pullup resistors, which work identically to pullup resistors on the digital pins. They are enabled by issuing a command such as 
    //  digitalWrite(A0, INPUT_PULLUP);  // set pullup on analog pin 0 
    //  Be aware however that turning on a pullup will affect the values reported by analogRead(). 
    
  //  Details and Caveats
    //  The analogRead command will not work correctly if a pin has been previously set to an output, so if this is the case, set it back to an input before using 
    //  analogRead. Similarly if the pin has been set to HIGH as an output, the pullup resistor will be set, when switched back to an input. 
    //  The Atmega datasheet also cautions against switching analog pins in close temporal proximity to making A/D readings (analogRead) on other analog pins.
    //  This can cause electrical noise and introduce jitter in the analog system. It may be desirable, after manipulating analog pins (in digital mode),
    //  to add a short delay before using analogRead() to read other analog pins.
  
 //  Note
 //  == comparison operator
 //  = assignment operator (sets it equal to)
  
 //  Comparison Operators:
 //   x == y (x is equal to y)
 //   x != y (x is not equal to y)
 //   x <  y (x is less than y)  
 //   x >  y (x is greater than y) 
 //   x <= y (x is less than or equal to y) 
 //   x >= y (x is greater than or equal to y)

 // LCD
    //  The circuit:
      // ArduinoPinA4 is the SDA pin for LCD display
      // ArduinoPinA5 is the SCL pin for LCD display
     

#include <ModbusRtu.h>                                          // Ensure that the ModbusRtu library is included (under libraries)
#include <Wire.h>                                               // Ensure that the Wire library is included (under libraries)
#include <LiquidCrystal_I2C.h>                                  // Ensure that the LiquidCrystal_I2C library is included (under libraries)
#include <Encoder.h>                                            // MPG encoder included


LiquidCrystal_I2C lcd(0x3F,20,4);                               // Set the LCD address to 0x3F for a 16 chars and 2 line display
                                                                // Check the address of the LCD by using I2C Scanner

#define ID   1                                                  // Slave ID no (1 to 276)                                         

Modbus slave(ID, 0, 0);                                         // This is the slave ID for RS-232 or USB-FTDI

    int8_t state = 0;                                           // This is the status according to au16data (is needed)
    unsigned long tempus;    

    int au16data[27];                                            // Data array (integers) for modbus network sharing (Register 00 to Register 26)


// Variables to store Register names
    
    // Pin 0 reserved for RX
    // Pin 1 reserved for TX
    
    const byte Register00 = 0;                                             // Holding Register for Pin 2
    const byte Register01 = 1;                                             // Holding Register for Pin 3
    const byte Register02 = 2;                                             // Holding Register for Pin 4
    const byte Register03 = 3;                                             // Holding Register for Pin 5 
    const byte Register04 = 4;                                             // Holding Register for Pin 6 
    const byte Register05 = 5;                                             // Holding Register for Pin 7 
    const byte Register06 = 6;                                             // Holding Register for Pin 8 
    const byte Register07 = 7;                                             // Holding Register for Pin 9 
    const byte Register08 = 8;                                             // Holding Register for Pin 10 
    const byte Register09 = 9;                                             // Holding Register for Pin 11
    const byte Register10 = 10;                                            // Holding Register for Pin 12
    const byte Register11 = 11;                                            // Arduino Register for Pin 13
                                                                   
    const byte Register12 = 12;                                            // Holding Register for Pin A0
    const byte Register13 = 13;                                            // Holding Register for Pin A1
    const byte Register14 = 14;                                            // Holding Register for Pin A2
    const byte Register15 = 15;                                            // Holding Register for Pin A3
    const byte Register16 = 16;                                            // Holding Register for Pin A4
    const byte Register17 = 17;                                            // Holding Register for Pin A5
        
    const byte Register18 = 18;                                            // Holding Register for LCD values interger X
    const byte Register19 = 19;                                            // Holding Register for LCD values decimal X
    const byte Register20 = 20;                                            // Holding Register for LCD values interger Y
    const byte Register21 = 21;                                            // Holding Register for LCD values decimal Y
    const byte Register22 = 22;                                            // Holding Register for LCD values interger Z
    const byte Register23 = 23;                                            // Holding Register for LCD values decimal Z

    const byte Register24 = 24;                                            // Holding Register for MPG1 values
    const byte Register25 = 25;                                            // Holding Register for MPG2 values   
    const byte Register26 = 26;                                            // Holding Register for MPG3 values 
        
    // const byte Register27 = 27;                                            // Holding Register for InCount
    // const byte Register28 = 28;                                            // Holding Register for OutCount
    // const byte Register29 = 29;                                            // Holding Register for ErrorCount


// Variables to store input or output types
     const byte DigitalInput = 0;                                          // Digital Input
     const byte DigitalOutput = 1;                                         // Digital Output
     const byte AnalogInput = 2;                                           // Analog Input
     const byte PWMOutput = 3;                                             // Pulse Width Modulation Output
     const byte LCDOutput = 4;                                             // Liquid Cristal Display Output
     const byte MPGInput = 5;                                              // Manual Pulse Generator Input


// Variables to store arduino pin types
  // Choose what you want each Arduino Pin to be then:
  // Edit the list below:
  // Ensure that you do not use one Arduino pin twice.
  // ***************** Change these to be either DigitalInputs, DigitalOuputs, AnalogInputs, PWMOutputs or MPGInput *******************
           
      // Arduino pin 0 is reserved for RX
      // Arduino pin 1 is reserved for TX
     int ArduinoPin02Type = MPGInput;                     // Use DigitalInput, DigitalOutput or MPGInput
     int ArduinoPin03Type = MPGInput;                     // Use DigitalInput, DigitalOutput, PWMOutput or MPGInput
     int ArduinoPin04Type = DigitalInput;                 // Use DigitalInput, DigitalOutput or MPGInput
     int ArduinoPin05Type = DigitalInput;                 // Use DigitalInput, DigitalOutput, PWMOutput or MPGInput
     int ArduinoPin06Type = DigitalInput;                 // Use DigitalInput, DigitalOutput, PWMOutput or MPGInput
     int ArduinoPin07Type = DigitalInput;                 // Use DigitalInput, DigitalOutput or MPGInput
     int ArduinoPin08Type = DigitalInput;                 // Use DigitalInput, DigitalOutput or MPGInput
     int ArduinoPin09Type = DigitalInput;                 // Use DigitalInput, DigitalOutput, PWMOutput or MPGInput
     int ArduinoPin10Type = DigitalInput;                 // Use DigitalInput, DigitalOutput, PWMOutput or MPGInput
     int ArduinoPin11Type = DigitalInput;                 // Use DigitalInput, DigitalOutput, PWMOutput or MPGInput
     int ArduinoPin12Type = DigitalInput;                 // Use DigitalInput, DigitalOutput or MPGInput
     int ArduinoPin13Type = DigitalInput;                 // Use DigitalInput, DigitalOutput or MPGInput

     int ArduinoPinA0Type = DigitalOutput;                // Use DigitalInput, DigitalOutput, AnalogInput or MPGInput
     int ArduinoPinA1Type = DigitalOutput;                // Use DigitalInput, DigitalOutput, AnalogInput or MPGInput
     int ArduinoPinA2Type = DigitalOutput;                // Use DigitalInput, DigitalOutput, AnalogInput or MPGInput
     int ArduinoPinA3Type = DigitalOutput;                // Use DigitalInput, DigitalOutput, AnalogInput or MPGInput

     int ArduinoPinA4Type = LCDOutput;                    // Use DigitalInput, DigitalOutput, AnalogInput or LCDOutput // A4 is shared with SDA if the LCD is used
     int ArduinoPinA5Type = LCDOutput;                    // Use DigitalInput, DigitalOutput, AnalogInput or LCDOutput // A5 is shared with SCL if the LCD is used


  // Variable to store LCD Available
    // *********** Change this to be either true or false *******************
     boolean LCDAvailable = true ;                             // Use true if LCD is available,or false if not available

     
// Variables to store pin names
  // Do not change these
      // Arduino Pin 0 is reserved for RX
      // Arduino Pin 1 is reserved for TX
     const byte ArduinoPin02 = 2;                                 // Arduino Pin 2
     const byte ArduinoPin03 = 3;                                 // Arduino Pin 3
     const byte ArduinoPin04 = 4;                                 // Arduino Pin 4
     const byte ArduinoPin05 = 5;                                 // Arduino Pin 5
     const byte ArduinoPin06 = 6;                                 // Arduino Pin 6
     const byte ArduinoPin07 = 7;                                 // Arduino Pin 7
     const byte ArduinoPin08 = 8;                                 // Arduino Pin 8
     const byte ArduinoPin09 = 9;                                 // Arduino Pin 9
     const byte ArduinoPin10 = 10;                                // Arduino Pin 10
     const byte ArduinoPin11 = 11;                                // Arduino Pin 11
     const byte ArduinoPin12 = 12;                                // Arduino Pin 12
     const byte ArduinoPin13 = 13;                                // Arduino Pin 13
    
     const byte ArduinoPinA0 = A0;                                // Arduino Pin A0
     const byte ArduinoPinA1 = A1;                                // Arduino Pin A1
     const byte ArduinoPinA2 = A2;                                // Arduino Pin A2
     const byte ArduinoPinA3 = A3;                                // Arduino Pin A3
     const byte ArduinoPinA4 = A4;                                // Arduino Pin A4
     const byte ArduinoPinA5 = A5;                                // Arduino Pin A5  

   
// Register Bits
    // Arranged from right(Bit 0) to left(Bit 15)
     const byte Bit0 = 0;


// Varables to store sensor values
     int SensorValueA0 = 0;
     int SensorValueA1 = 0;    
     int SensorValueA2 = 0;    
     int SensorValueA3 = 0;
     int SensorValueA4 = 0;
     int SensorValueA5 = 0;


 // Varables to store output values    
     int MappedSensorValueA0 = 0;
     int MappedSensorValueA1 = 0;   
     int MappedSensorValueA2 = 0;     
     int MappedSensorValueA3 = 0;
     int MappedSensorValueA4 = 0;
     int MappedSensorValueA5 = 0;


// Change these two numbers to the pins connected to your encoder.
  //   Best Performance: both pins have interrupt capability
  //   Good Performance: only the first pin has interrupt capability
  //   Low Performance:  neither pin has interrupt capability
       Encoder MPG1(2, 3);                            // Use 2 and 3 for MPG1
       // Encoder MPG2(4, 5);                            // Use 4 and 5 for MPG2
       // Encoder MPG3(6, 7);                            // Use 6 and 7 for MPG3     
  //   Avoid using pins with LEDs attached


// MPG Old Positions
    int MPG1oldPosition = 0;
    // int MPG2oldPosition = 0;
    // int MPG3oldPosition = 0;

         
void setup() {  

 // Pin modes for Inputs and outputs
 
    // Arduino pin 0 is reserved for RX
      
    // Arduino pin 1 is reserved for TX
      
    // Arduino pin 2
      if (ArduinoPin02Type == DigitalInput) { pinMode(ArduinoPin02, INPUT_PULLUP); }
      else if (ArduinoPin02Type == DigitalOutput) { pinMode(ArduinoPin02, OUTPUT); }
      // ArduinoPin02Type can not be an AnalogInput
      // ArduinoPin02Type can not be an PWMOutput
      else if (ArduinoPin02Type == MPGInput) { pinMode(ArduinoPin02, INPUT_PULLUP); } 
      
    // Arduino pin 3
      if (ArduinoPin03Type == DigitalInput) { pinMode(ArduinoPin03, INPUT_PULLUP); }
      else if (ArduinoPin03Type == DigitalOutput) { pinMode(ArduinoPin03, OUTPUT); }
      // ArduinoPin03Type can not be an AnalogInput
      else if (ArduinoPin03Type == PWMOutput) { pinMode(ArduinoPin03, OUTPUT); } 
      else if (ArduinoPin03Type == MPGInput) { pinMode(ArduinoPin03, INPUT_PULLUP); } 
            
    // Arduino pin 4
      if (ArduinoPin04Type == DigitalInput) { pinMode(ArduinoPin04, INPUT_PULLUP); }
      else if (ArduinoPin04Type == DigitalOutput) { pinMode(ArduinoPin04, OUTPUT); }
      // ArduinoPin04Type can not be an AnalogInput
      // ArduinoPin04Type can not be an PWMOutput
      else if (ArduinoPin04Type == MPGInput) { pinMode(ArduinoPin04, INPUT_PULLUP); } 
      
    // Arduino pin 5
      if (ArduinoPin05Type == DigitalInput) { pinMode(ArduinoPin05, INPUT_PULLUP); }
      else if (ArduinoPin05Type == DigitalOutput) { pinMode(ArduinoPin05, OUTPUT); }
      // ArduinoPin05Type can not be an AnalogInput
      else if (ArduinoPin05Type == PWMOutput) { pinMode(ArduinoPin05, OUTPUT); } 
      else if (ArduinoPin05Type == MPGInput) { pinMode(ArduinoPin05, INPUT_PULLUP); } 
      
    // Arduino pin 6
      if (ArduinoPin06Type == DigitalInput) { pinMode(ArduinoPin06, INPUT_PULLUP); }
      else if (ArduinoPin06Type == DigitalOutput) { pinMode(ArduinoPin06, OUTPUT); }
      // ArduinoPin06Type can not be an AnalogInput
      else if (ArduinoPin06Type == PWMOutput) { pinMode(ArduinoPin06, OUTPUT); }
      else if (ArduinoPin06Type == MPGInput) { pinMode(ArduinoPin06, INPUT_PULLUP); }  
      
    // Arduino pin 7
      if (ArduinoPin07Type == DigitalInput) { pinMode(ArduinoPin07, INPUT_PULLUP); }
      else if (ArduinoPin07Type == DigitalOutput) { pinMode(ArduinoPin07, OUTPUT); }
      // ArduinoPin07Type can not be an AnalogInput
      // ArduinoPin07Type can not be an PWMOutput
      else if (ArduinoPin07Type == MPGInput) { pinMode(ArduinoPin07, INPUT_PULLUP); } 
      
    // Arduino pin 8
      if (ArduinoPin08Type == DigitalInput) { pinMode(ArduinoPin08, INPUT_PULLUP); }
      else if (ArduinoPin08Type == DigitalOutput) { pinMode(ArduinoPin08, OUTPUT); }
      // ArduinoPin08Type can not be an AnalogInput
      // ArduinoPin08Type can not be an PWMOutput 
      else if (ArduinoPin08Type == MPGInput) { pinMode(ArduinoPin08, INPUT_PULLUP); } 
      
    // Arduino pin 9
      if (ArduinoPin09Type == DigitalInput) { pinMode(ArduinoPin09, INPUT_PULLUP); }
      else if (ArduinoPin09Type == DigitalOutput) { pinMode(ArduinoPin09, OUTPUT); }
      // ArduinoPin09Type can not be an AnalogInput
      else if (ArduinoPin09Type == PWMOutput) { pinMode(ArduinoPin09, OUTPUT); } 
      else if (ArduinoPin09Type == MPGInput) { pinMode(ArduinoPin09, INPUT_PULLUP); } 
      
    // Arduino pin 10
      if (ArduinoPin10Type == DigitalInput) { pinMode(ArduinoPin10, INPUT_PULLUP); }
      else if (ArduinoPin10Type == DigitalOutput) { pinMode(ArduinoPin10, OUTPUT); }
      // ArduinoPin10Type can not be an AnalogInput
      else if (ArduinoPin10Type == PWMOutput) { pinMode(ArduinoPin10, OUTPUT); } 
      else if (ArduinoPin10Type == MPGInput) { pinMode(ArduinoPin10, INPUT_PULLUP); } 

    // Arduino pin 11
      if (ArduinoPin11Type == DigitalInput) { pinMode(ArduinoPin11, INPUT_PULLUP); }
      else if (ArduinoPin11Type == DigitalOutput) { pinMode(ArduinoPin11, OUTPUT); }
      // ArduinoPin11Type can not be an AnalogInput
      else if (ArduinoPin11Type == PWMOutput) { pinMode(ArduinoPin11, OUTPUT); } 
      else if (ArduinoPin11Type == MPGInput) { pinMode(ArduinoPin11, INPUT_PULLUP); } 

    // Arduino pin 12
      if (ArduinoPin12Type == DigitalInput) { pinMode(ArduinoPin12, INPUT_PULLUP); }
      else if (ArduinoPin12Type == DigitalOutput) { pinMode(ArduinoPin12, OUTPUT); }
      // ArduinoPin12Type can not be an AnalogInput
      // ArduinoPin12Type can not be an PWMOutput
      else if (ArduinoPin12Type == MPGInput) { pinMode(ArduinoPin12, INPUT_PULLUP); } 

    // Arduino pin 13   
      if (ArduinoPin13Type == DigitalInput) { pinMode(ArduinoPin13, INPUT_PULLUP); }
      else if (ArduinoPin13Type == DigitalOutput) { pinMode(ArduinoPin13, OUTPUT); }
      // ArduinoPin13Type can not be an AnalogInput
      // ArduinoPin13Type can not be an PWMOutput
      else if (ArduinoPin13Type == MPGInput) { pinMode(ArduinoPin13, INPUT_PULLUP); } 


    // Arduino pin A0       
      if (ArduinoPinA0Type == DigitalInput) { pinMode(ArduinoPinA0, INPUT_PULLUP); }
      else if (ArduinoPinA0Type == DigitalOutput) { pinMode(ArduinoPinA0, OUTPUT); }
      else if (ArduinoPinA0Type == AnalogInput) { pinMode(ArduinoPinA0, INPUT); }
      // ArduinoPinA0Type can not be a PWMOutput
      else if (ArduinoPinA0Type == MPGInput) { pinMode(ArduinoPinA0, INPUT_PULLUP); } 
      
    // Arduino pin A1
      if (ArduinoPinA1Type == DigitalInput) { pinMode(ArduinoPinA1, INPUT_PULLUP); }
      else if (ArduinoPinA1Type == DigitalOutput) { pinMode(ArduinoPinA1, OUTPUT); }
      else if (ArduinoPinA1Type == AnalogInput) { pinMode(ArduinoPinA1, INPUT); }
      // ArduinoPinA1Type can not be a PWMOutput
      else if (ArduinoPinA1Type == MPGInput) { pinMode(ArduinoPinA1, INPUT_PULLUP); } 
            
    // Arduino pin A2
      if (ArduinoPinA2Type == DigitalInput) { pinMode(ArduinoPinA2, INPUT_PULLUP); }
      else if (ArduinoPinA2Type == DigitalOutput) { pinMode(ArduinoPinA2, OUTPUT); }
      else if (ArduinoPinA2Type == AnalogInput) { pinMode(ArduinoPinA2, INPUT); }
      // ArduinoPinA2Type can not be a PWMOutput
      else if (ArduinoPinA2Type == MPGInput) { pinMode(ArduinoPinA2, INPUT_PULLUP); } 
      
    // Arduino pin A3
      if (ArduinoPinA3Type == DigitalInput) { pinMode(ArduinoPinA3, INPUT_PULLUP); }
      else if (ArduinoPinA3Type == DigitalOutput) { pinMode(ArduinoPinA3, OUTPUT); }
      else if (ArduinoPinA3Type == AnalogInput) { pinMode(ArduinoPinA3, INPUT); }
      // ArduinoPinA3Type can not be a PWMOutput
      else if (ArduinoPinA3Type == MPGInput) { pinMode(ArduinoPinA3, INPUT_PULLUP); } 
      
    // Arduino pin A4
      if (ArduinoPinA4Type == DigitalInput) { pinMode(ArduinoPinA4, INPUT_PULLUP); }
      else if (ArduinoPinA4Type == DigitalOutput) { pinMode(ArduinoPinA4, OUTPUT); }
      else if (ArduinoPinA4Type == AnalogInput) { pinMode(ArduinoPinA4, INPUT); }
      // ArduinoPinA4Type can not be a PWMOutput
      else if (ArduinoPinA4Type == MPGInput) { pinMode(ArduinoPinA4, INPUT_PULLUP); } 
      
    // Arduino pin A5
      if (ArduinoPinA5Type == DigitalInput) { pinMode(ArduinoPinA5, INPUT_PULLUP); }
      else if (ArduinoPinA5Type == DigitalOutput) { pinMode(ArduinoPinA5, OUTPUT); }
      else if (ArduinoPinA5Type == AnalogInput) { pinMode(ArduinoPinA5, INPUT); }
      // ArduinoPinA5Type can not be a PWMOutput
      else if (ArduinoPinA5Type == MPGInput) { pinMode(ArduinoPinA5, INPUT_PULLUP); } 
 
      
// Reset all output pins to Low or Zero

    // Arduino pin 0 is reserved for RX
      
    // Arduino pin 1 is reserved for TX
      
    // Arduino pin 2
      if (ArduinoPin02Type == DigitalOutput) { digitalWrite(ArduinoPin02, LOW ); }
      // Arduino pin 02 is not an PWM output pin
      
    // Arduino pin 3
      if (ArduinoPin03Type == DigitalOutput) { digitalWrite(ArduinoPin03, LOW ); }
      else if (ArduinoPin03Type == PWMOutput) { analogWrite(ArduinoPin03, 0 ); }         // use Zero
      
    // Arduino pin 4
      if (ArduinoPin04Type == DigitalOutput) { digitalWrite(ArduinoPin04, LOW ); }
      // Arduino pin 04 is not an PWM output pin
      
    // Arduino pin 5
      if (ArduinoPin05Type == DigitalOutput) { digitalWrite(ArduinoPin05, LOW ); }
      else if (ArduinoPin05Type == PWMOutput) { analogWrite(ArduinoPin05, 0 ); }
      
    // Arduino pin 6
      if (ArduinoPin06Type == DigitalOutput) { digitalWrite(ArduinoPin06, LOW ); }
      else if (ArduinoPin06Type == PWMOutput) { analogWrite(ArduinoPin06, 0 ); }
      
    // Arduino pin 7
      if (ArduinoPin07Type == DigitalOutput) { digitalWrite(ArduinoPin07, LOW ); }
      // Arduino pin 07 is not an PWM output pin 
      
    // Arduino pin 8
      if (ArduinoPin08Type == DigitalOutput) { digitalWrite(ArduinoPin08, LOW ); }
      // Arduino pin 08 is not an PWM output pin   
      
    // Arduino pin 9
      if (ArduinoPin09Type == DigitalOutput) { digitalWrite(ArduinoPin09, LOW ); }
      else if (ArduinoPin09Type == PWMOutput) { analogWrite(ArduinoPin09, 0 ); }
      
    // Arduino pin 10
      if (ArduinoPin10Type == DigitalOutput) { digitalWrite(ArduinoPin10, LOW ); }
      else if (ArduinoPin10Type == PWMOutput) { analogWrite(ArduinoPin10, 0 ); }
      
    // Arduino pin 11
      if (ArduinoPin11Type == DigitalOutput) { digitalWrite(ArduinoPin11, LOW ); }
      else if (ArduinoPin11Type == PWMOutput) { analogWrite(ArduinoPin11, 0 ); }
      
    // Arduino pin 12
      if (ArduinoPin12Type == DigitalOutput) { digitalWrite(ArduinoPin12, LOW ); }
      // Arduino pin 12 is not an PWM output pin  
      
    // Arduino pin 13
      if (ArduinoPin13Type == DigitalOutput) { digitalWrite(ArduinoPin13, LOW ); }
      // Arduino pin 13 is not an PWM output pin

      
    // Arduino pin A0
      if (ArduinoPinA0Type == DigitalOutput) { digitalWrite(ArduinoPinA0, LOW ); }
      // ArduinoPinA0Type can not be an PWMOutput
      
    // Arduino pin A1
      if (ArduinoPinA1Type == DigitalOutput) { digitalWrite(ArduinoPinA1, LOW ); }
      // ArduinoPinA1Type can not be an PWMOutput
            
    // Arduino pin A2
      if (ArduinoPinA2Type == DigitalOutput) { digitalWrite(ArduinoPinA2, LOW ); }
      // ArduinoPinA2Type can not be an PWMOutput
            
    // Arduino pin A3
      if (ArduinoPinA3Type == DigitalOutput) { digitalWrite(ArduinoPinA3, LOW ); }
      // ArduinoPinA3Type can not be an PWMOutput
            
    // Arduino pin A4
      if (ArduinoPinA4Type == DigitalOutput) { digitalWrite(ArduinoPinA4, LOW ); }
      // ArduinoPinA4Type can not be an PWMOutput
   
    // Arduino pin A5
      if (ArduinoPinA5Type == DigitalOutput) { digitalWrite(ArduinoPinA5, LOW ); }
      // ArduinoPinA5Type can not be an PWMOutput


  // Start communication
      slave.begin( 19200 );                                    // Start cmmunication as a slave Modbus


  // Liquid Cristal Display
      if (LCDAvailable = true) {
        
          lcd.init();                                          // Initialize the lcd 
          lcd.setBacklight(50);                                // Switch the backlight on (0 to 255)   
          lcd.setCursor(1,0);                                  // Set cursor to (Colum 1, Row 0)              
          lcd.print("March3 Pendant");                         // Print a message to the LCD.
          lcd.setCursor(1,1);                                  // Set cursor to (Colum 1, Row 1)
          lcd.print("using Arduino");                          // Print a message to the LCD.
          lcd.setCursor(1,2);                                  // Set cursor to (Colum 1, Row 2)              
          lcd.print(" by: B Bouwer");                          // Print a message to the LCD.
          lcd.setCursor(1,3);                                  // Set cursor to (Colum 1, Row 3)
          lcd.print("     2018");                              // Print a message to the LCD.
          
          lcd.noBlink();                                       // Set the cursor not to blink
          lcd.noCursor();                                      // Remove the cursor
          delay (2000);                                        // Wait a bit
          
          // Scroll 20 positions (string length + display length) to the right to move it offscreen right:
          for (int positionCounter = 0; positionCounter < 20; positionCounter++) {
            lcd.scrollDisplayRight();                          // Scroll one position right:
            delay(250);                                        // Wait a bit
          }  // End of for        

          delay(250);
          lcd.clear ();                                        // Clears the LCD
          
      }   // End of if (LCDAvailable = true) {

  
} // End of void setup() {



void loop() {

  state = slave.poll( au16data, 27 );                          // Poll the state from the master (is needed)

  // Note - bracets{} are used in this section after the if (Variable == Amount){do something; do something}
      
    // Arduino pin 0 is reserved for RX
      
    // Arduino pin 1 is reserved for TX

    // Arduino pin 2          
      if (ArduinoPin02Type == DigitalInput){ bitWrite( au16data[Register00], Bit0, not digitalRead(ArduinoPin02)); }     // Use NOT to invert the value
      else if (ArduinoPin02Type == DigitalOutput) { digitalWrite( ArduinoPin02, bitRead( au16data[Register00], Bit0 )); }
      // ArduinoPin02Type can not be an analog input
      // ArduinoPin02Type can not be an PWM output
       
    // Arduino pin 3
      if (ArduinoPin03Type == DigitalInput){ bitWrite( au16data[Register01], Bit0, not digitalRead(ArduinoPin03)); }
      else if (ArduinoPin03Type == DigitalOutput) { digitalWrite( ArduinoPin03, bitRead( au16data[Register01], Bit0 )); }  
      // ArduinoPin03Type can not be an analog input
      else if (ArduinoPin03Type == PWMOutput) { analogWrite( ArduinoPin03, au16data[Register01] ); }
 
    // Arduino pin 4
      if (ArduinoPin04Type == DigitalInput){ bitWrite( au16data[Register02], Bit0, not digitalRead(ArduinoPin04)); }
      else if (ArduinoPin04Type == DigitalOutput) { digitalWrite( ArduinoPin04, bitRead( au16data[Register02], Bit0 )); }
      // ArduinoPin04Type can not be an analog input
      // ArduinoPin04Type can not be an PWM output 

    // Arduino pin 5
      if (ArduinoPin05Type == DigitalInput){ bitWrite( au16data[Register03], Bit0, not digitalRead(ArduinoPin05)); }
      else if (ArduinoPin05Type == DigitalOutput) { digitalWrite( ArduinoPin05, bitRead( au16data[Register03], Bit0 )); }
      // ArduinoPin05Type can not be an analog input
      else if (ArduinoPin05Type == PWMOutput) { analogWrite( ArduinoPin05, au16data[Register03] ); }    

    // Arduino pin 6
      if (ArduinoPin06Type == DigitalInput){ bitWrite( au16data[Register04], Bit0, not digitalRead(ArduinoPin06)); }
      else if (ArduinoPin06Type == DigitalOutput) { digitalWrite( ArduinoPin06, bitRead( au16data[Register04], Bit0 )); }
      // ArduinoPin06Type can not be an analog input
      else if (ArduinoPin06Type == PWMOutput) { analogWrite( ArduinoPin06, au16data[Register04] ); }

    // Arduino pin 7
      if (ArduinoPin07Type == DigitalInput){ bitWrite( au16data[Register05], Bit0, not digitalRead(ArduinoPin07)); }
      else if (ArduinoPin07Type == DigitalOutput) { digitalWrite( ArduinoPin07, bitRead( au16data[Register05], Bit0 )); }
      // ArduinoPin07Type can not be an analog input
      // ArduinoPin07Type can not be an PWM output 

    // Arduino pin 8
      if (ArduinoPin08Type == DigitalInput){ bitWrite( au16data[Register06], Bit0, not digitalRead(ArduinoPin08)); }
      else if (ArduinoPin08Type == DigitalOutput) { digitalWrite( ArduinoPin08, bitRead( au16data[Register06], Bit0 )); }
      // ArduinoPin08Type can not be an analog input
      // ArduinoPin08Type can not be an PWM output   

    // Arduino pin 9
      if (ArduinoPin09Type == DigitalInput){ bitWrite( au16data[Register07], Bit0, not digitalRead(ArduinoPin09)); }
      else if (ArduinoPin09Type == DigitalOutput) { digitalWrite( ArduinoPin09, bitRead( au16data[Register07], Bit0 )); }
      // ArduinoPin09Type can not be an analog input
      else if (ArduinoPin09Type == PWMOutput) { analogWrite( ArduinoPin09, au16data[Register07] ); }
      
    // Arduino pin 10
      if (ArduinoPin10Type == DigitalInput){ bitWrite( au16data[Register08], Bit0, not digitalRead(ArduinoPin10)); }
      else if (ArduinoPin10Type == DigitalOutput) { digitalWrite( ArduinoPin10, bitRead( au16data[Register08], Bit0 )); }
      // ArduinoPin10Type can not be an analog input
      else if (ArduinoPin10Type == PWMOutput) { analogWrite( ArduinoPin10, au16data[Register08] ); }

    // Arduino pin 11
      if (ArduinoPin11Type == DigitalInput){ bitWrite( au16data[Register09], Bit0, not digitalRead(ArduinoPin11)); }
      else if (ArduinoPin11Type == DigitalOutput) { digitalWrite( ArduinoPin11, bitRead( au16data[Register09], Bit0 )); }
      // ArduinoPin11Type can not be an analog input
      else if (ArduinoPin11Type == PWMOutput) { analogWrite( ArduinoPin11, au16data[Register09] ); }

    // Arduino pin 12
      if (ArduinoPin12Type == DigitalInput){ bitWrite( au16data[Register10], Bit0, not digitalRead(ArduinoPin12)); }
      else if (ArduinoPin12Type == DigitalOutput) { digitalWrite( ArduinoPin12, bitRead( au16data[Register10], Bit0 )); }
      // ArduinoPin12Type can not be an analog input
      // ArduinoPin12Type can not be an PWM output 

    // Arduino pin 13
      if (ArduinoPin13Type == DigitalInput){ bitWrite( au16data[Register11], Bit0, not digitalRead(ArduinoPin13)); }
      else if (ArduinoPin13Type == DigitalOutput) { digitalWrite( ArduinoPin13, bitRead( au16data[Register11], Bit0 )); }
      // ArduinoPin13Type can not be an analog input
      // ArduinoPin13Type can not be an PWM output

    // On the analog inputs the sensor values must be maped to usable values for PWM outputs (eg. 0 to 255)  
    // Arduino pin A0
      if (ArduinoPinA0Type == DigitalInput){ bitWrite( au16data[Register12], Bit0, not digitalRead(ArduinoPinA0)); }
      else if (ArduinoPinA0Type == DigitalOutput) { digitalWrite( ArduinoPinA0, bitRead( au16data[Register12], Bit0 )); }
      else if (ArduinoPinA0Type == AnalogInput) { SensorValueA0 = analogRead(ArduinoPinA0); MappedSensorValueA0 = map(SensorValueA0, 0, 1023, 0, 255); au16data[Register12] = MappedSensorValueA0; }
      // ArduinoPinA0Type can not be an PWM output

    // Arduino pin A1   
      if (ArduinoPinA1Type == DigitalInput){ bitWrite( au16data[Register13], Bit0, not digitalRead(ArduinoPinA1)); }
      else if (ArduinoPinA1Type == DigitalOutput) { digitalWrite( ArduinoPinA1, bitRead( au16data[Register13], Bit0 )); }
      else if (ArduinoPinA1Type == AnalogInput) { SensorValueA1 = analogRead(ArduinoPinA1); MappedSensorValueA1 = map(SensorValueA1, 0, 1023, 0, 255); au16data[Register13] = MappedSensorValueA1; }
      // ArduinoPinA1Type can not be an PWM output

    // Arduino pin A2
      if (ArduinoPinA2Type == DigitalInput){ bitWrite( au16data[Register14], Bit0, not digitalRead(ArduinoPinA2)); }
      else if (ArduinoPinA2Type == DigitalOutput) { digitalWrite( ArduinoPinA2, bitRead( au16data[Register14], Bit0 )); }
      else if (ArduinoPinA2Type == AnalogInput) { SensorValueA2 = analogRead(ArduinoPinA2); MappedSensorValueA2 = map(SensorValueA2, 0, 1023, 0, 255); au16data[Register14] = MappedSensorValueA2; }
      // ArduinoPinA2Type can not be an PWM output

    // Arduino pin A3
      if (ArduinoPinA3Type == DigitalInput){ bitWrite( au16data[Register15], Bit0, not digitalRead(ArduinoPinA3)); }
      else if (ArduinoPinA3Type == DigitalOutput) { digitalWrite( ArduinoPinA3, bitRead( au16data[Register15], Bit0 )); }
      else if (ArduinoPinA3Type == AnalogInput) { SensorValueA3 = analogRead(ArduinoPinA3); MappedSensorValueA3 = map(SensorValueA3, 0, 1023, 0, 255); au16data[Register15] = MappedSensorValueA3; }
      // ArduinoPinA3Type can not be an PWM output

    // Arduino pin A4
      if (ArduinoPinA4Type == DigitalInput){ bitWrite( au16data[Register16], Bit0, not digitalRead(ArduinoPinA4)); }   
      else if (ArduinoPinA4Type == DigitalOutput) { digitalWrite( ArduinoPinA4, bitRead( au16data[Register16], Bit0 )); }
      else if (ArduinoPinA4Type == AnalogInput) { SensorValueA4 = analogRead(ArduinoPinA4); MappedSensorValueA4 = map(SensorValueA4, 0, 1023, 0, 255); au16data[Register16] = MappedSensorValueA4; }
      // ArduinoPinA4Type can not be an PWM output

    // Arduino pin A5             
      if (ArduinoPinA5Type == DigitalInput){ bitWrite( au16data[Register17], Bit0, not digitalRead(ArduinoPinA5)); }  
      else if (ArduinoPinA5Type == DigitalOutput) { digitalWrite( ArduinoPinA5, bitRead( au16data[Register17], Bit0 )); } 
      else if (ArduinoPinA5Type == AnalogInput) { SensorValueA5 = analogRead(ArduinoPinA5); MappedSensorValueA5 = map(SensorValueA5, 0, 1023, 0, 255); au16data[Register17] = MappedSensorValueA5; }
      // ArduinoPinA5Type can not be an PWM output


  // Liquid Cristal Display
     // The 16 bits in the registers are only big enough to store Binary: 1111111111111111 or Decimal: 65535 max
     // The numbers in the Mach3 DRO's consists of intergers and decimals (eg: 9999.6666)
     // To solve this we use two registers for each axis DRO.
        // One register for everything to the left of the decimal point (the interger 9999)
        // One register for everything to the right of the decimal point (the decimal 6666)

        // Use a brain in Mach3 to split the DRO position of each axis into two portions
           // DRO --> Formula --> Modbus Register
              // Formula1 = int(A)                           // This is the interger (left part)
              // Formula2 = (A * 10000) - (int(A) * 10000)   // This is the decimal (right part)
              // Use arduino to combine the values again

     if (LCDAvailable == true) {
        lcd.noBlink();                                       // Set the cursor not to blink
        lcd.noCursor();                                      // Remove the cursor
        
     // LCD X position      
          lcd.setCursor(0, 0);             // (Colum, Line)
          lcd.print("X:"); 
          
       // Interger position X
          
          int IntergerXpos = au16data[Register18];
          
          if (IntergerXpos >= 0    and IntergerXpos <= 9 )   {lcd.print("   +");}
          if (IntergerXpos >= 10   and IntergerXpos <= 99 )  {lcd.print("  +") ;}       
          if (IntergerXpos >= 100  and IntergerXpos <= 999 ) {lcd.print(" +")  ;}
          if (IntergerXpos >= 1000 and IntergerXpos <= 9999 ){lcd.print("+")   ;}                     
          
          if (IntergerXpos <= -1   and IntergerXpos >= -9 )  {lcd.print("   ") ;}
          if (IntergerXpos <= -10  and IntergerXpos >= -99 ) {lcd.print("  ")  ;}       
          if (IntergerXpos <= -100 and IntergerXpos >= -999 ){lcd.print(" ")   ;}      
          
          lcd.print(IntergerXpos);


       // Decimal Point X
          lcd.print(".");         
                   
       // Decimal position X
          
          int DecimalXpos = au16data[Register19];
          
          if (DecimalXpos < 0 and IntergerXpos == 0 ) {lcd.setCursor(2, 0); lcd.print("   -");}           

          lcd.setCursor(8, 0);
          
          if (DecimalXpos < 0) {DecimalXpos = DecimalXpos * -1 ;}
          if (DecimalXpos >= 0   and DecimalXpos <= 9 )  {lcd.print("000");}
          if (DecimalXpos >= 10  and DecimalXpos <= 99 ) {lcd.print("00") ;}
          if (DecimalXpos >= 100 and DecimalXpos <= 999 ){lcd.print("0")  ;}      
          
          lcd.print(DecimalXpos);
          lcd.print(" "); 

                  
     // LCD Y position      
          lcd.setCursor(0, 1);             // (Colum, Line)
          lcd.print("Y:"); 
          
       // Interger position Y
          
          int IntergerYpos = au16data[Register20];
          
          if (IntergerYpos >= 0    and IntergerYpos <= 9 )   {lcd.print("   +");}
          if (IntergerYpos >= 10   and IntergerYpos <= 99 )  {lcd.print("  +") ;}       
          if (IntergerYpos >= 100  and IntergerYpos <= 999 ) {lcd.print(" +")  ;}
          if (IntergerYpos >= 1000 and IntergerYpos <= 9999 ){lcd.print("+")   ;}
            
          if (IntergerYpos <= -1   and IntergerYpos >= -9 )  {lcd.print("   ") ;}
          if (IntergerYpos <= -10  and IntergerYpos >= -99 ) {lcd.print("  ")  ;}       
          if (IntergerYpos <= -100 and IntergerYpos >= -999 ){lcd.print(" ")   ;}      
          
          lcd.print(IntergerYpos);

       // Decimal Point Y
          lcd.print(".");         
                   
       // Decimal position Y
          int DecimalYpos = au16data[Register21];
           
          if (DecimalYpos < 0 and IntergerYpos == 0 ) {lcd.setCursor(2, 1); lcd.print("   -");}
          
          lcd.setCursor(8, 1);
                      
          if (DecimalYpos < 0) {DecimalYpos = DecimalYpos * -1 ;}
          if (DecimalYpos >= 0   and DecimalYpos <= 9 )  {lcd.print("000");}
          if (DecimalYpos >= 10  and DecimalYpos <= 99 ) {lcd.print("00") ;}
          if (DecimalYpos >= 100 and DecimalYpos <= 999 ){lcd.print("0")  ;}      
          
          lcd.print(DecimalYpos);
          lcd.print(" ");    

               
     // LCD Z position      
          lcd.setCursor(0, 2);             // (Colum, Line)
          lcd.print("Z:"); 
          
       // Interger position Z
          int IntergerZpos = au16data[Register22];
          
          if (IntergerZpos >= 0    and IntergerZpos <= 9 )   {lcd.print("   +");}
          if (IntergerZpos >= 10   and IntergerZpos <= 99 )  {lcd.print("  +") ;}       
          if (IntergerZpos >= 100  and IntergerZpos <= 999 ) {lcd.print(" +")  ;}
          if (IntergerZpos >= 1000 and IntergerZpos <= 9999 ){lcd.print("+")   ;}   
                             
          if (IntergerZpos <= -1   and IntergerZpos >= -9 )  {lcd.print("   ") ;}
          if (IntergerZpos <= -10  and IntergerZpos >= -99 ) {lcd.print("  ")  ;}       
          if (IntergerZpos <= -100 and IntergerZpos >= -999 ){lcd.print(" ")   ;}      
          
          lcd.print(IntergerZpos);

       // Decimal Point Z
          lcd.print(".");         
                   
       // Decimal position Z
          int DecimalZpos = au16data[Register23];
 
          if (DecimalZpos < 0 and IntergerZpos == 0 ) {lcd.setCursor(2, 2); lcd.print("   -");}  
         
          lcd.setCursor(8, 2);
           
          if (DecimalZpos < 0) {DecimalZpos = DecimalZpos * -1 ;}
          if (DecimalZpos >= 0   and DecimalZpos <= 9 )  {lcd.print("000");}
          if (DecimalZpos >= 10  and DecimalZpos <= 99 ) {lcd.print("00") ;}
          if (DecimalZpos >= 100 and DecimalZpos <= 999 ){lcd.print("0")  ;}      
          lcd.print(DecimalZpos);
          lcd.print(" ");         


 
     }  // End of if(LCDAvailable == true) {



  // Manual Pulse Generator 1: Register24  
     if (ArduinoPin02Type == MPGInput and ArduinoPin03Type == MPGInput) {
       int MPG1newPosition = MPG1.read();

        if (MPG1newPosition != MPG1oldPosition) {
          MPG1oldPosition = MPG1newPosition;
          int MPG1displayedPosition = MPG1newPosition/-4;
          au16data[Register24] = MPG1displayedPosition;
        } // End of if 
 
     } // End of if 

  /*
  // Manual Pulse Generator 2: Register25
     if (ArduinoPin04Type == MPGInput and ArduinoPin05Type == MPGInput) {
       int MPG2newPosition = MPG2.read();

        if (MPG2newPosition != MPG2oldPosition) {
          MPG2oldPosition = MPG2newPosition;
          int MPG2displayedPosition = MPG2newPosition/-4;
          au16data[Register25] = MPG2displayedPosition;
        } // End of if 
 
     } // End of if
  */

   
  /* 
  // Manual Pulse Generator 3: Register26 
     if (ArduinoPin06Type == MPGInput and ArduinoPin07Type == MPGInput) {
       int MPG3newPosition = MPG3.read();

        if (MPG3newPosition != MPG3oldPosition) {
          MPG3oldPosition = MPG3newPosition;
          int MPG3displayedPosition = MPG3newPosition/-4;
          au16data[Register26] = MPG3displayedPosition;
        } // End of if 
 
     } // End of if 
  */

  /*
  // Diagnose communication
      au16data[Register27] = slave.getInCnt();                           // Not used now
      au16data[Register28] = slave.getOutCnt();                          // Not used now
      au16data[Register29] = slave.getErrCnt();                          // Not used now
  */


// Print all of the register values to the serial monitor

    Serial.begin(19200);                               // To make Serial Monitor work
     
    Serial.print("0:");  Serial.print(au16data[Register00]); Serial.print(" "); 
    Serial.print("1:");  Serial.print(au16data[Register01]); Serial.print(" "); 
    Serial.print("2:");  Serial.print(au16data[Register02]); Serial.print(" "); 
    Serial.print("3:");  Serial.print(au16data[Register03]); Serial.print(" "); 
    Serial.print("4:");  Serial.print(au16data[Register04]); Serial.print(" "); 
    Serial.print("5:");  Serial.print(au16data[Register05]); Serial.print(" "); 
    Serial.print("6:");  Serial.print(au16data[Register06]); Serial.print(" "); 
    Serial.print("7:");  Serial.print(au16data[Register07]); Serial.print(" ");
    Serial.print("8:");  Serial.print(au16data[Register08]); Serial.print(" "); 
    Serial.print("9:");  Serial.print(au16data[Register09]); Serial.print(" ");
    Serial.print("10:"); Serial.print(au16data[Register10]); Serial.print(" ");
    Serial.print("11:"); Serial.print(au16data[Register11]); Serial.print(" ");
    Serial.print("12:"); Serial.print(au16data[Register12]); Serial.print(" ");  
    Serial.print("13:"); Serial.print(au16data[Register13]); Serial.print(" ");
    Serial.print("14:"); Serial.print(au16data[Register14]); Serial.print(" ");               
    Serial.print("15:"); Serial.print(au16data[Register15]); Serial.print(" ");
    Serial.print("16:"); Serial.print(au16data[Register16]); Serial.print(" ");
    Serial.print("17:"); Serial.print(au16data[Register17]); Serial.print(" ");
    Serial.print("18:"); Serial.print(au16data[Register18]); Serial.print(" ");
    Serial.print("19:"); Serial.print(au16data[Register19]); Serial.print(" ");
    Serial.print("20:"); Serial.print(au16data[Register20]); Serial.print(" ");
    Serial.print("21:"); Serial.print(au16data[Register21]); Serial.print(" ");
    Serial.print("22:"); Serial.print(au16data[Register22]); Serial.print(" ");
    Serial.print("23:"); Serial.print(au16data[Register23]); Serial.print(" ");
    Serial.print("24:"); Serial.print(au16data[Register24]); Serial.print(" ");
    Serial.print("25:"); Serial.print(au16data[Register25]); Serial.print(" ");
    Serial.print("26:"); Serial.print(au16data[Register26]); Serial.print(" ");
    // Serial.print("27:"); Serial.print(au16data[Register27]); Serial.print(" ");
    // Serial.print("28:"); Serial.print(au16data[Register28]); Serial.print(" ");
    // Serial.print("29:"); Serial.print(au16data[Register29]); Serial.print(" "); 
   
    Serial.println();

      
} // End of void loop


// End of program
